@@ -25,7 +25,7 @@ type_qq = function(distribution = qnorm) {
   }
 
   draw_qq = function() {
-    fun = function(ix, iy, icol, ibg, ipch, ilwd, ilty, cex, xlab, ...) {
+    fun = function(ix, iy, icol, ibg, ipch, ilwd, ilty, icex, xlab, ...) {
       points(
         x = ix,
         y = iy,
@@ -34,14 +34,14 @@ type_qq = function(distribution = qnorm) {
         type = "p",
         pch = ipch,
         lwd = ilwd,
-        cex = cex
+        cex = icex
       )
 
       if (!is.null(ilty)) {
-        iy <- quantile(iy, c(0.25, 0.75))
-        ix <- quantile(ix, c(0.25, 0.75))
-        slope <- diff(iy) / diff(ix)
-        intercept <- iy[1] - slope * ix[1]
+        iy = quantile(iy, c(0.25, 0.75))
+        ix = quantile(ix, c(0.25, 0.75))
+        slope = diff(iy) / diff(ix)
+        intercept = iy[1] - slope * ix[1]
         abline(a = intercept, b = slope, lty = ilty, col = icol, lwd = ilwd)
       }
     }
