@@ -9,6 +9,8 @@
 #' @param xpd Logical value or `NA` denoting text clipping behaviour, following
 #'  [graphics::par()].
 #' @param srt Numeric giving the desired string rotation in degrees.
+#' @param clim Numeric giving the lower and upper limits of the character
+#'   expansion (`cex`) normalization for bubble charts.
 #' @inheritParams graphics::text
 #' @examples
 #' tinyplot(mpg ~ hp | factor(cyl),
@@ -19,7 +21,7 @@
 #'     adj = 0
 #'   )
 #' )
-#' 
+#'
 #' # to avoid clipping text at the plot region, we can use xpd = NA
 #' tinyplot(mpg ~ hp | factor(cyl),
 #'   data = mtcars,
@@ -32,24 +34,51 @@
 #' )
 #'
 #' @export
-type_text = function(labels, adj = NULL, pos = NULL, offset = 0.5, vfont = NULL, font = NULL, xpd = NULL, srt = 0) {
+type_text = function(labels, adj = NULL, pos = NULL, offset = 0.5, vfont = NULL, font = NULL, xpd = NULL, srt = 0, clim = c(0.5, 2.5)) {
   out = list(
     draw = draw_text(adj = adj, pos = pos, offset = offset, vfont = vfont, font = font, xpd = xpd, srt = srt),
-    data = data_text(labels = labels),
+    data = data_text(labels = labels, clim = clim),
     name = "text"
   )
   class(out) = "tinyplot_type"
   return(out)
 }
 
-data_text = function(labels) {
-  fun = function(datapoints, ...) {
+data_text = function(labels, clim = c(0.5, 2.5)) {
+  fun = function(datapoints, legend_args, cex = NULL, ...) {
     if (length(labels) != 1 && length(labels) != nrow(datapoints)) {
-      msg <- sprintf("`labels` must be of length 1 or %s.", nrow(datapoints))
+      msg = sprintf("`labels` must be of length 1 or %s.", nrow(datapoints))
       stop(msg, call. = FALSE)
     }
     datapoints$labels = labels
-    out = list(datapoints = datapoints)
+    if (is.factor(datapoints$x)) datapoints$x = as.numeric(datapoints$x)
+    if (is.factor(datapoints$y)) datapoints$y = as.numeric(datapoints$y)
+
+    # browser()
+    bubble = FALSE
+    bubble_cex = 1
+    if (!is.null(cex) && length(cex) == nrow(datapoints)) {
+      bubble = TRUE
+      ## Identify the pretty break points for our bubble labels
+      bubble_labs = pretty(cex, n = 5)
+      len_labs = length(bubble_labs)
+      # cex = rescale_num(c(bubble_labs, cex), to = clim)
+      cex = rescale_num(sqrt(c(bubble_labs, cex)) / pi, to = clim)
+      bubble_cex = cex[1:len_labs]
+      cex = cex[(len_labs + 1):length(cex)]
+      names(bubble_cex) = format(bubble_labs)
+      if (max(clim) > 2.5) {
+        legend_args[["x.intersp"]] = max(clim) / 2.5
+        legend_args[["y.intersp"]] = sapply(bubble_cex / 2.5, max, 1)
+      }
+    }
+
+    out = list(
+      datapoints = datapoints,
+      cex = cex,
+      bubble = bubble,
+      bubble_cex = bubble_cex
+    )
     return(out)
   }
   return(fun)
@@ -57,14 +86,14 @@ data_text = function(labels) {
 
 draw_text = function(adj = NULL, pos = NULL, offset = 0.5, vfont = NULL, font = NULL, xpd = NULL, srt = 0) {
   if (is.null(xpd)) xpd = par("xpd")
-  fun = function(ix, iy, ilabels, icol, cex, ...) {
+  fun = function(ix, iy, ilabels, icol, icex, ...) {
     text(
       x = ix, y = iy, labels = ilabels, col = icol,
       adj = adj, pos = pos, offset = offset,
       vfont = vfont, font = font,
       xpd = xpd,
       srt = srt,
-      cex = cex
+      cex = icex
     )
   }
 }
