@@ -1,7 +1,7 @@
-rescale_num = function (x, from = NULL, to = NULL) {
+rescale_num = function(x, from = NULL, to = NULL) {
   if (is.null(from)) from = range(x)
   if (is.null(to)) to = c(0, 1)
-  (x - from[1])/diff(from) * diff(to) + to[1]
+  (x - from[1]) / diff(from) * diff(to) + to[1]
 }
 
 ## Function for efficiently checking whether a vector has more than n unique
@@ -31,21 +31,38 @@ more_than_n_unique = function(x, n, small_vec_len = 1e3L) {
 
 ## Null coalescing operator
 if (getRversion() <= "4.4.0") {
-   `%||%` = function(x, y) if (is.null(x)) y else x
+  `%||%` = function(x, y) if (is.null(x)) y else x
 }
 
 
 ## Function that computes an appropriate bandwidth kernel based on a string
 ## input
 bw_fun = function(kernel, x) {
-    kernel = tolower(kernel)
-    switch(
-        kernel,
-        nrd0 = bw.nrd0(x),
-        nrd  = bw.nrd(x),
-        ucv  = bw.ucv(x),
-        bcv  = bw.bcv(x),
-        sj   = bw.SJ(x),
-        stop("Invalid `bw` string. Choose from 'nrd0', 'nrd', 'ucv', 'bcv', or 'SJ'.")
-    )
+  kernel = tolower(kernel)
+  switch(kernel,
+    nrd0 = bw.nrd0(x),
+    nrd  = bw.nrd(x),
+    ucv  = bw.ucv(x),
+    bcv  = bw.bcv(x),
+    sj   = bw.SJ(x),
+    stop("Invalid `bw` string. Choose from 'nrd0', 'nrd', 'ucv', 'bcv', or 'SJ'.")
+  )
+}
+
+
+swap_variables = function(env, ...) {
+  pairs = list(...)
+  for (p in pairs) {
+    tmp = get(p[1], envir = env)
+    assign(p[1], get(p[2], envir = env), envir = env)
+    assign(p[2], tmp, envir = env)
+  }
+}
+
+swap_columns = function(dp, a, b) {
+  va = dp[[a]]
+  vb = dp[[b]]
+  dp[[a]] = if (!is.null(vb)) vb else NULL
+  dp[[b]] = if (!is.null(va)) va else NULL
+  dp
 }
